@@ -1,5 +1,5 @@
 /* Definitions of Tensilica's Xtensa target machine for GNU compiler.
-   Copyright 2001,2002,2003,2004 Free Software Foundation, Inc.
+   Copyright 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
    Contributed by Bob Wilson (bwilson@tensilica.com) at Tensilica.
 
 This file is part of GCC.
@@ -16,15 +16,14 @@ for more details.
 
 You should have received a copy of the GNU General Public License
 along with GCC; see the file COPYING.  If not, write to the Free
-Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.  */
+Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301, USA.  */
 
 /* Get Xtensa configuration settings */
 #include "xtensa-config.h"
 
 /* Standard GCC variables that we reference.  */
 extern int current_function_calls_alloca;
-extern int target_flags;
 extern int optimize;
 
 /* External variables defined in xtensa.c.  */
@@ -42,10 +41,6 @@ extern struct rtx_def * branch_cmp[2];	/* operands for compare */
 extern enum cmp_type branch_type;	/* what type of branch to use */
 extern unsigned xtensa_current_frame_size;
 
-/* Masks for the -m switches */
-#define MASK_NO_FUSED_MADD	0x00000001	/* avoid f-p mul/add */
-#define MASK_CONST16		0x00000002	/* use CONST16 instruction */
-
 /* Macros used in the machine description to select various Xtensa
    configuration options.  */
 #define TARGET_BIG_ENDIAN	XCHAL_HAVE_BE
@@ -66,40 +61,23 @@ extern unsigned xtensa_current_frame_size;
 #define TARGET_ABS		XCHAL_HAVE_ABS
 #define TARGET_ADDX		XCHAL_HAVE_ADDX
 
-/* Macros controlled by command-line options.  */
-#define TARGET_NO_FUSED_MADD	(target_flags & MASK_NO_FUSED_MADD)
-#define TARGET_CONST16		(target_flags & MASK_CONST16)
-
 #define TARGET_DEFAULT (						\
   (XCHAL_HAVE_L32R	? 0 : MASK_CONST16))
 
-#define TARGET_SWITCHES							\
-{									\
-  {"const16",			MASK_CONST16,				\
-    N_("Use CONST16 instruction to load constants")},			\
-  {"no-const16",		-MASK_CONST16,				\
-    N_("Use PC-relative L32R instruction to load constants")},		\
-  {"no-fused-madd",		MASK_NO_FUSED_MADD,			\
-    N_("Disable fused multiply/add and multiply/subtract FP instructions")}, \
-  {"fused-madd",		-MASK_NO_FUSED_MADD,			\
-    N_("Enable fused multiply/add and multiply/subtract FP instructions")}, \
-  {"text-section-literals",	0,					\
-    N_("Intersperse literal pools with code in the text section")},	\
-  {"no-text-section-literals",	0,					\
-    N_("Put literal pools in a separate literal section")},		\
-  {"target-align",		0,					\
-    N_("Automatically align branch targets to reduce branch penalties")}, \
-  {"no-target-align",		0,					\
-    N_("Do not automatically align branch targets")},			\
-  {"longcalls",			0,					\
-    N_("Use indirect CALLXn instructions for large programs")},		\
-  {"no-longcalls",		0,					\
-    N_("Use direct CALLn instructions for fast calls")},		\
-  {"",				TARGET_DEFAULT, 0}			\
-}
-
-
 #define OVERRIDE_OPTIONS override_options ()
+
+/* Reordering blocks for Xtensa is not a good idea unless the compiler
+   understands the range of conditional branches.  Currently all branch
+   relaxation for Xtensa is handled in the assembler, so GCC cannot do a
+   good job of reordering blocks.  Do not enable reordering unless it is
+   explicitly requested.  */
+#define OPTIMIZATION_OPTIONS(LEVEL, SIZE)				\
+  do									\
+    {									\
+      flag_reorder_blocks = 0;						\
+    }									\
+  while (0)
+
 
 /* Target CPU builtins.  */
 #define TARGET_CPU_CPP_BUILTINS()					\
@@ -108,6 +86,7 @@ extern unsigned xtensa_current_frame_size;
     builtin_assert ("machine=xtensa");					\
     builtin_define ("__xtensa__");					\
     builtin_define ("__XTENSA__");					\
+    builtin_define ("__XTENSA_WINDOWED_ABI__");				\
     builtin_define (TARGET_BIG_ENDIAN ? "__XTENSA_EB__" : "__XTENSA_EL__"); \
     if (!TARGET_HARD_FLOAT)						\
       builtin_define ("__XTENSA_SOFT_FLOAT__");				\
@@ -162,7 +141,6 @@ extern unsigned xtensa_current_frame_size;
 #define INT_TYPE_SIZE 32
 #define SHORT_TYPE_SIZE 16
 #define LONG_TYPE_SIZE 32
-#define MAX_LONG_TYPE_SIZE 32
 #define LONG_LONG_TYPE_SIZE 64
 #define FLOAT_TYPE_SIZE 32
 #define DOUBLE_TYPE_SIZE 64
@@ -205,19 +183,19 @@ extern unsigned xtensa_current_frame_size;
       }									\
   } while (0)
 
-/* The promotion described by `PROMOTE_MODE' should also be done for
-   outgoing function arguments.  */
-#define PROMOTE_FUNCTION_ARGS
-
-/* The promotion described by `PROMOTE_MODE' should also be done for
-   the return value of functions.  Note: `FUNCTION_VALUE' must perform
-   the same promotions done by `PROMOTE_MODE'.  */
-#define PROMOTE_FUNCTION_RETURN
-
 /* Imitate the way many other C compilers handle alignment of
    bitfields and the structures that contain them.  */
 #define PCC_BITFIELD_TYPE_MATTERS 1
 
+/* Disable the use of word-sized or smaller complex modes for structures,
+   and for function arguments in particular, where they cause problems with
+   register a7.  The xtensa_copy_incoming_a7 function assumes that there is
+   a single reference to an argument in a7, but with small complex modes the
+   real and imaginary components may be extracted separately, leading to two
+   uses of the register, only one of which would be replaced.  */
+#define MEMBER_TYPE_FORCES_BLK(FIELD, MODE) \
+  ((MODE) == CQImode || (MODE) == CHImode)
+
 /* Align string constants and constructors to at least a word boundary.
    The typical use of this macro is to increase alignment for string
    constants to be word aligned so that 'strcpy' calls that copy
@@ -240,10 +218,6 @@ extern unsigned xtensa_current_frame_size;
 	|| TREE_CODE (TYPE) == UNION_TYPE				\
 	|| TREE_CODE (TYPE) == RECORD_TYPE)) ? BITS_PER_WORD : (ALIGN))
 
-/* An argument declared as 'char' or 'short' in a prototype should
-   actually be passed as an 'int'.  */
-#define PROMOTE_PROTOTYPES 1
-
 /* Operations between registers always perform the operation
    on the full register even if a narrower mode is specified.  */
 #define WORD_REGISTER_OPERATIONS
@@ -426,17 +400,10 @@ extern char xtensa_hard_regno_mode_ok[][FIRST_PSEUDO_REGISTER];
    a real pain to get them reloaded.  */
 #define FPCC_REGNUM (BR_REG_FIRST + 0)
 
-/* Pass structure value address as an "invisible" first argument.  */
-#define STRUCT_VALUE 0
-
 /* It is as good or better to call a constant function address than to
    call an address kept in a register.  */
 #define NO_FUNCTION_CSE 1
 
-/* It is as good or better for a function to call itself with an
-   explicit address than to call an address kept in a register.  */
-#define NO_RECURSIVE_FUNCTION_CSE 1
-
 /* Xtensa processors have "register windows".  GCC does not currently
    take advantage of the possibility for variable-sized windows; instead,
    we use a fixed window size of 8.  */
@@ -556,29 +523,16 @@ extern enum reg_class xtensa_char_to_class[256];
 
    For Xtensa:
 
-   I = 12-bit signed immediate for movi
-   J = 8-bit signed immediate for addi
+   I = 12-bit signed immediate for MOVI
+   J = 8-bit signed immediate for ADDI
    K = 4-bit value in (b4const U {0})
    L = 4-bit value in b4constu
-   M = 7-bit value in simm7
-   N = 8-bit unsigned immediate shifted left by 8 bits for addmi
-   O = 4-bit value in ai4const
-   P = valid immediate mask value for extui */
+   M = 7-bit immediate value for MOVI.N
+   N = 8-bit unsigned immediate shifted left by 8 bits for ADDMI
+   O = 4-bit immediate for ADDI.N
+   P = valid immediate mask value for EXTUI */
 
-#define CONST_OK_FOR_LETTER_P(VALUE, C)					\
-  ((C) == 'I' ? (xtensa_simm12b (VALUE))				\
-   : (C) == 'J' ? (xtensa_simm8 (VALUE))				\
-   : (C) == 'K' ? (((VALUE) == 0) || xtensa_b4const (VALUE))		\
-   : (C) == 'L' ? (xtensa_b4constu (VALUE))				\
-   : (C) == 'M' ? (xtensa_simm7 (VALUE))				\
-   : (C) == 'N' ? (xtensa_simm8x256 (VALUE))				\
-   : (C) == 'O' ? (xtensa_ai4const (VALUE))				\
-   : (C) == 'P' ? (xtensa_mask_immediate (VALUE))			\
-   : FALSE)
-
-
-/* Similar, but for floating constants, and defining letters G and H.
-   Here VALUE is the CONST_DOUBLE rtx itself.  */
+#define CONST_OK_FOR_LETTER_P  xtensa_const_ok_for_letter_p
 #define CONST_DOUBLE_OK_FOR_LETTER_P(VALUE, C) (0)
 
 
@@ -602,15 +556,7 @@ extern enum reg_class xtensa_char_to_class[256];
    address will be checked anyway because of the code in
    GO_IF_LEGITIMATE_ADDRESS.  */
 
-#define EXTRA_CONSTRAINT(OP, CODE)					\
-  ((GET_CODE (OP) != MEM) ?						\
-       ((CODE) >= 'R' && (CODE) <= 'U'					\
-	&& reload_in_progress && GET_CODE (OP) == REG			\
-        && REGNO (OP) >= FIRST_PSEUDO_REGISTER)				\
-   : ((CODE) == 'R') ? smalloffset_mem_p (OP)				\
-   : ((CODE) == 'T') ? !TARGET_CONST16 && constantpool_mem_p (OP)	\
-   : ((CODE) == 'U') ? !constantpool_mem_p (OP)				\
-   : FALSE)
+#define EXTRA_CONSTRAINT  xtensa_extra_constraint
 
 #define PREFERRED_RELOAD_CLASS(X, CLASS)				\
   xtensa_preferred_reload_class (X, CLASS, 0)
@@ -655,12 +601,17 @@ extern enum reg_class xtensa_char_to_class[256];
 #define INITIAL_ELIMINATION_OFFSET(FROM, TO, OFFSET)			\
   do {									\
     compute_frame_size (get_frame_size ());				\
-    if ((FROM) == FRAME_POINTER_REGNUM)					\
-      (OFFSET) = 0;							\
-    else if ((FROM) == ARG_POINTER_REGNUM)				\
-      (OFFSET) = xtensa_current_frame_size;				\
-    else								\
-      abort ();								\
+    switch (FROM)							\
+      {									\
+      case FRAME_POINTER_REGNUM:					\
+        (OFFSET) = 0;							\
+	break;								\
+      case ARG_POINTER_REGNUM:						\
+        (OFFSET) = xtensa_current_frame_size;				\
+	break;								\
+      default:								\
+	gcc_unreachable ();						\
+      }									\
   } while (0)
 
 /* If defined, the maximum amount of space required for outgoing
@@ -702,20 +653,10 @@ extern enum reg_class xtensa_char_to_class[256];
 /* Don't worry about compatibility with PCC.  */
 #define DEFAULT_PCC_STRUCT_RETURN 0
 
-/* For Xtensa, up to 4 words can be returned in registers.  (It would
-   have been nice to allow up to 6 words in registers but GCC cannot
-   support that.  The return value must be given one of the standard
-   MODE_INT modes, and there is no 6 word mode.  Instead, if we try to
-   return a 6 word structure, GCC selects the next biggest mode
-   (OImode, 8 words) and then the register allocator fails because
-   there is no 8-register group beginning with a10.)  */
-#define RETURN_IN_MEMORY(TYPE)						\
-  ((unsigned HOST_WIDE_INT) int_size_in_bytes (TYPE) > 4 * UNITS_PER_WORD)
-
 /* Define how to find the value returned by a library function
    assuming the value has mode MODE.  Because we have defined
-   PROMOTE_FUNCTION_RETURN, we have to perform the same promotions as
-   PROMOTE_MODE.  */
+   TARGET_PROMOTE_FUNCTION_RETURN that returns true, we have to
+   perform the same promotions as PROMOTE_MODE.  */
 #define XTENSA_LIBCALL_VALUE(MODE, OUTGOINGP)				\
   gen_rtx_REG ((GET_MODE_CLASS (MODE) == MODE_INT			\
 		&& GET_MODE_SIZE (MODE) < UNITS_PER_WORD)		\
@@ -792,9 +733,6 @@ typedef struct xtensa_args
 #define FUNCTION_INCOMING_ARG(CUM, MODE, TYPE, NAMED) \
   function_arg (&CUM, MODE, TYPE, TRUE)
 
-/* Arguments are never passed partly in memory and partly in registers.  */
-#define FUNCTION_ARG_PARTIAL_NREGS(CUM, MODE, TYPE, NAMED) (0)
-
 /* Specify function argument alignment.  */
 #define FUNCTION_ARG_BOUNDARY(MODE, TYPE)				\
   ((TYPE) != 0								\
@@ -805,22 +743,6 @@ typedef struct xtensa_args
       ? PARM_BOUNDARY							\
       : GET_MODE_ALIGNMENT (MODE)))
 
-/* Nonzero if we do not know how to pass TYPE solely in registers.
-   We cannot do so in the following cases:
-
-   - if the type has variable size
-   - if the type is marked as addressable (it is required to be constructed
-     into the stack)
-
-   This differs from the default in that it does not check if the padding
-   and mode of the type are such that a copy into a register would put it
-   into the wrong part of the register.  */
-
-#define MUST_PASS_IN_STACK(MODE, TYPE)					\
-  ((TYPE) != 0								\
-   && (TREE_CODE (TYPE_SIZE (TYPE)) != INTEGER_CST			\
-       || TREE_ADDRESSABLE (TYPE)))
-
 /* Profiling Xtensa code is typically done with the built-in profiling
    feature of Tensilica's instruction set simulator, which does not
    require any compiler support.  Profiling code on a real (i.e.,
@@ -871,7 +793,7 @@ typedef struct xtensa_args
 
 #define TRAMPOLINE_TEMPLATE(STREAM)					\
   do {									\
-    fprintf (STREAM, "\t.begin no-generics\n");				\
+    fprintf (STREAM, "\t.begin no-transform\n");			\
     fprintf (STREAM, "\tentry\tsp, %d\n", MIN_FRAME_SIZE);		\
 									\
     /* save the return address */					\
@@ -909,11 +831,13 @@ typedef struct xtensa_args
     /* jump to the instruction following the entry */			\
     fprintf (STREAM, "\taddi\ta8, a8, 3\n");				\
     fprintf (STREAM, "\tjx\ta8\n");					\
-    fprintf (STREAM, "\t.end no-generics\n");				\
+    fprintf (STREAM, "\t.byte\t0\n");					\
+    fprintf (STREAM, "\t.end no-transform\n");				\
   } while (0)
 
-/* Size in bytes of the trampoline, as an integer.  */
-#define TRAMPOLINE_SIZE 59
+/* Size in bytes of the trampoline, as an integer.  Make sure this is
+   a multiple of TRAMPOLINE_ALIGNMENT to avoid -Wpadded warnings.  */
+#define TRAMPOLINE_SIZE 60
 
 /* Alignment required for trampolines, in bits.  */
 #define TRAMPOLINE_ALIGNMENT (32)
@@ -924,26 +848,14 @@ typedef struct xtensa_args
     rtx addr = ADDR;							\
     emit_move_insn (gen_rtx_MEM (SImode, plus_constant (addr, 12)), CHAIN); \
     emit_move_insn (gen_rtx_MEM (SImode, plus_constant (addr, 16)), FUNC); \
-    emit_library_call (gen_rtx (SYMBOL_REF, Pmode, "__xtensa_sync_caches"), \
+    emit_library_call (gen_rtx_SYMBOL_REF (Pmode, "__xtensa_sync_caches"), \
 		       0, VOIDmode, 1, addr, Pmode);			\
   } while (0)
 
-/* If defined, is a C expression that produces the machine-specific
-   code for a call to '__builtin_saveregs'.  This code will be moved
-   to the very beginning of the function, before any parameter access
-   are made.  The return value of this function should be an RTX that
-   contains the value to use as the return of '__builtin_saveregs'.  */
-#define EXPAND_BUILTIN_SAVEREGS \
-  xtensa_builtin_saveregs
-
 /* Implement `va_start' for varargs and stdarg.  */
 #define EXPAND_BUILTIN_VA_START(valist, nextarg) \
   xtensa_va_start (valist, nextarg)
 
-/* Implement `va_arg'.  */
-#define EXPAND_BUILTIN_VA_ARG(valist, type) \
-  xtensa_va_arg (valist, type)
-
 /* If defined, a C expression that produces the machine-specific code
    to setup the stack so that arbitrary frames can be accessed.
 
@@ -977,8 +889,7 @@ typedef struct xtensa_args
    when count == 0 and the stack pointer when count > 0.  */
 
 #define DYNAMIC_CHAIN_ADDRESS(frame)					\
-  gen_rtx (PLUS, Pmode, frame,						\
-	   gen_rtx_CONST_INT (VOIDmode, -3 * UNITS_PER_WORD))
+  gen_rtx_PLUS (Pmode, frame, GEN_INT (-3 * UNITS_PER_WORD))
 
 /* Define this if the return address of a particular stack frame is
    accessed from the frame pointer of the previous stack frame.  */
@@ -1096,7 +1007,8 @@ typedef struct xtensa_args
    operand on the target machine when generating position independent
    code.  */
 #define LEGITIMATE_PIC_OPERAND_P(X)					\
-  ((GET_CODE (X) != SYMBOL_REF || SYMBOL_REF_LOCAL_P (X))		\
+  ((GET_CODE (X) != SYMBOL_REF						\
+    || (SYMBOL_REF_LOCAL_P (X) && !SYMBOL_REF_EXTERNAL_P (X)))		\
    && GET_CODE (X) != LABEL_REF						\
    && GET_CODE (X) != CONST)
 
@@ -1123,10 +1035,10 @@ typedef struct xtensa_args
 	    && xtensa_simm8x256 (INTVAL (plus1) & ~0xff))		\
 	  {								\
 	    rtx temp = gen_reg_rtx (Pmode);				\
-	    emit_insn (gen_rtx (SET, Pmode, temp,			\
-				gen_rtx (PLUS, Pmode, plus0,		\
+	    emit_insn (gen_rtx_SET (Pmode, temp,			\
+				gen_rtx_PLUS (Pmode, plus0,		\
 					 GEN_INT (INTVAL (plus1) & ~0xff)))); \
-	    (X) = gen_rtx (PLUS, Pmode, temp,				\
+	    (X) = gen_rtx_PLUS (Pmode, temp,				\
 			   GEN_INT (INTVAL (plus1) & 0xff));		\
 	    goto WIN;							\
 	  }								\
@@ -1152,11 +1064,6 @@ typedef struct xtensa_args
    for the index in the tablejump instruction.  */
 #define CASE_VECTOR_MODE (SImode)
 
-/* Define this if the tablejump instruction expects the table
-   to contain offsets from the address of the table.
-   Do not define this if the table should contain absolute addresses.  */
-/* #define CASE_VECTOR_PC_RELATIVE */
-
 /* Define this as 1 if 'char' should by default be signed; else as 0.  */
 #define DEFAULT_SIGNED_CHAR 0
 
@@ -1206,37 +1113,6 @@ typedef struct xtensa_args
 
 #define BRANCH_COST 3
 
-/* Optionally define this if you have added predicates to
-   'MACHINE.c'.  This macro is called within an initializer of an
-   array of structures.  The first field in the structure is the
-   name of a predicate and the second field is an array of rtl
-   codes.  For each predicate, list all rtl codes that can be in
-   expressions matched by the predicate.  The list should have a
-   trailing comma.  */
-
-#define PREDICATE_CODES							\
-  {"add_operand",		{ REG, CONST_INT, SUBREG }},		\
-  {"arith_operand",		{ REG, CONST_INT, SUBREG }},		\
-  {"nonimmed_operand",		{ REG, SUBREG, MEM }},			\
-  {"mem_operand",		{ MEM }},				\
-  {"mask_operand",		{ REG, CONST_INT, SUBREG }},		\
-  {"extui_fldsz_operand",	{ CONST_INT }},				\
-  {"sext_fldsz_operand",	{ CONST_INT }},				\
-  {"lsbitnum_operand",		{ CONST_INT }},				\
-  {"fpmem_offset_operand",	{ CONST_INT }},				\
-  {"sext_operand",		{ REG, SUBREG, MEM }},			\
-  {"branch_operand",		{ REG, CONST_INT, SUBREG }},		\
-  {"ubranch_operand",		{ REG, CONST_INT, SUBREG }},		\
-  {"call_insn_operand",		{ CONST_INT, CONST, SYMBOL_REF, REG }},	\
-  {"move_operand",		{ REG, SUBREG, MEM, CONST_INT, CONST_DOUBLE, \
-				  CONST, SYMBOL_REF, LABEL_REF }},	\
-  {"const_float_1_operand",	{ CONST_DOUBLE }},			\
-  {"branch_operator",		{ EQ, NE, LT, GE }},			\
-  {"ubranch_operator",		{ LTU, GEU }},				\
-  {"boolean_operator",		{ EQ, NE }},
-
-/* Control the assembler format that we output.  */
-
 /* How to refer to registers in assembler output.
    This sequence is indexed by compiler's hard-register-number (see above).  */
 #define REGISTER_NAMES							\
